% ham_eom_FDR.m
% Program to compute the natural frequencies, mode shapes, and frequency response of the HAM isolation table
% Jonas Waterman, High Precision Devices, 1/12/2007
% Modified
% Updated with current design values at time of FDR, 3/26/2007
%%
%mods by Brian Lantz, Jan 4, 2007
% setup state-space model
% also calculate the tilt-horizontal coupling zero
% clear; 
close all;
% System parameters
fzz = f; % (Hz), Uncoupled natural frequency
mu = m; % (kg), stage 1 plus "unsuspended" payload mass
ms = 0; % (kg), "suspended" payload mass
mtot = mu+ms; % Total static mass
Jxx = 0.21; % (kg*m^2), polar moment of inertia about x (at CG)
Jyy = 0.21; % (kg*m^2), polar moment of inertia about y (at CG)
Jzz = 0.42; % (kg*m^2), polar moment of inertia about z (at CG)
g = 9.81; % (m/s^2), gravitational acceleration
% lf = .2032; % (m), Flexure total length
% lz = .1482; % (m), Flexure effective length
df = d; % (m), Flexure diameter
hf = -(0.4*0.0254+ZMP/1000*2+0.04215); % (m), Stage 1 plus payload CG offset above flexure LZMP
rs = .28341; % (m), Spring tip radius
%E = 1.876E11; % (N/m^2), Elastic modulus
nu = .25; % Poisson's ratio
G = Ef/(2*(1+nu)); % (N/m^2), Shear modulus
Jf = pi*df^4/32; % (m^4), Flexure area polar moment of inertia
hs = 0; % (m), Suspended payload height above table



Kzz = mu*(2*pi*fzz)^2/3; % (N/m), Spring stiffness
Kxx = mtot*g/(3*lz); % (N/m), Flexure stiffness
Kyy = Kxx;
Krxrx = (3/2)*Kzz*rs^2 + 3*Kxx*hf^2 - mu*g*hf - ms*g*hs;
Kryry = Krxrx;
Krzrz = 3/2^.5*(Kxx^2+Kyy^2)^.5*rs^2+3*G*Jf/lf;
% Assemble equation of motion matrices and solve eigenproblem
M = diag([mu mu mu Jxx Jyy Jzz]);
K = [3*Kxx 0 0 0 3*Kxx*hf 0;
0 3*Kyy 0 3*Kyy*hf 0 0;
0 0 3*Kzz 0 0 0;
0 3*Kyy*hf 0 Krxrx 0 0;
3*Kxx*hf 0 0 0 Kryry 0;
0 0 0 0 0 Krzrz];
[V, D] = eig(inv(M)*K);
Natural_frequencies = D.^.5./(2*pi)
Mode_shapes = 1e-5*round(1e5*V) % truncate the numeric junk
Response_variables = ['X ';'Y ';'Z ';'RX';'RY';'RZ']
num_DOF = length(M);
num_states = 2*num_DOF;
num_inputs = length(M); % the inputs are forces and torques
damp = 50*eye(size(M)); % lightly damped
F = [1 0 0 0 0 0;
0 1 0 0 0 0;
0 0 1 0 0 0;
0 hf 0 1 0 0;
hf 0 0 0 1 0;
0 0 0 0 0 1];
A = [zeros(num_DOF) eye(num_DOF);
-inv(M)*K -inv(M)*damp];
B = [zeros(num_DOF,num_inputs); inv(M)*F];
C = eye(num_DOF, num_states); % returns the first 2 states (x and ry);
D = zeros(num_DOF, num_inputs);
sys = ss(A, B, C, D);
freq = logspace(-1,1,1000);
w = 2*pi*freq;
sys_resp = freqresp(sys, w);
in.Fx = 1; in.Fy = 2; in.Fz = 3; in.Trx = 4; in.Try = 5; in.Trz = 6;
out.x = 1; out.y = 2; out.z = 3; out.rx = 4; out.ry = 5; out.rz = 6;
figure
pp=loglog(freq,3*Kxx*abs(squeeze(sys_resp(out.x,in.Fx,:))),'b',...
freq,3*Kzz*abs(squeeze(sys_resp(out.z,in.Fz,:))),'g',...
freq,Kryry*abs(squeeze(sys_resp(out.ry,in.Try,:))),'m',...
freq,Krzrz*abs(squeeze(sys_resp(out.rz,in.Trz,:))),'k',...
freq,Kryry/hf*abs(squeeze(sys_resp(out.ry,in.Fx,:))),'c');
grid on
set(pp(1),'LineWidth',2);
set(pp(2),'LineWidth',2);
set(pp(3),'LineWidth',2);
set(pp(4),'LineWidth',2);
set(pp(5),'LineWidth',2);
xlabel('freq (Hz)')
ylabel('normalized response magnitude (m/N or rad/N-m)')
title('System response, normalized by diagonal stiffness')
legend('Fx->x','Fz->z','Try->ry','Trz->rz', 'Fx->ry')
%%%%%%%%%%
% Calculate the tilt-horizontal coupling zero (Brian Lantz)
act_offset = 1e-3; % m offset of horz acts and LZMP
tilt_horz_coupling_zero = 1/(2*pi) * ...
sqrt(act_offset * g * 3*Kxx/Kryry)
% this is the frequency at which the horz seismometers
% see as much signal from the tilt (theta) of the stage as from
% the translation (x) of the stage.
% x = F / (3*Kxx) (total horz stiffness is 3*Kxx)
% theta = (F*act_offset) / Kryry
% set (g/w^2) * theta = x, and solve for w
% we can also set f = 2*pi*w = 50 mHz, and solve for
% the ratio of the total horizontal stiffness over the rotational stiffness
% (.05*2*pi)^2/(1e-3*9.81) = 10.061 m^-2